@@ -200,12 +200,80 @@ def _get_default(x, default):
         return default
 
 
+class OrderedSet(collections.MutableSet):
+    """
+    Standard Python OrderedSet recipe found at http://code.activestate.com/recipes/576694/
+
+    Modified to include a peek function to get the last element
+    """
+
+    def __init__(self, iterable=None):
+        self.end = end = []
+        end += [None, end, end]         # sentinel node for doubly linked list
+        self.map = {}                   # key --> [key, prev, next]
+        if iterable is not None:
+            self |= iterable
+
+    def __len__(self):
+        return len(self.map)
+
+    def __contains__(self, key):
+        return key in self.map
+
+    def add(self, key):
+        if key not in self.map:
+            end = self.end
+            curr = end[1]
+            curr[2] = end[1] = self.map[key] = [key, curr, end]
+
+    def discard(self, key):
+        if key in self.map:
+            key, prev, next = self.map.pop(key)
+            prev[2] = next
+            next[1] = prev
+
+    def __iter__(self):
+        end = self.end
+        curr = end[2]
+        while curr is not end:
+            yield curr[0]
+            curr = curr[2]
+
+    def __reversed__(self):
+        end = self.end
+        curr = end[1]
+        while curr is not end:
+            yield curr[0]
+            curr = curr[1]
+
+    def peek(self, last=True):
+        if not self:
+            raise KeyError('set is empty')
+        key = self.end[1][0] if last else self.end[2][0]
+        return key
+
+    def pop(self, last=True):
+        key = self.peek(last)
+        self.discard(key)
+        return key
+
+    def __repr__(self):
+        if not self:
+            return '%s()' % (self.__class__.__name__,)
+        return '%s(%r)' % (self.__class__.__name__, list(self))
+
+    def __eq__(self, other):
+        if isinstance(other, OrderedSet):
+            return len(self) == len(other) and list(self) == list(other)
+        return set(self) == set(other)
+
+
 class Task(object):
     def __init__(self, task_id, status, deps, resources=None, priority=0, family='', module=None,
                  params=None, tracking_url=None, status_message=None, retry_policy='notoptional'):
         self.id = task_id
         self.stakeholders = set()  # workers ids that are somehow related to this task (i.e. don't prune while any of these workers are still active)
-        self.workers = set()  # workers ids that can perform task - task is 'BROKEN' if none of these workers are active
+        self.workers = OrderedSet()  # workers ids that can perform task - task is 'BROKEN' if none of these workers are active
         if deps is None:
             self.deps = set()
         else:
@@ -464,7 +532,7 @@ class SimpleTaskState(object):
             elif task.scheduler_disable_time is not None and new_status != DISABLED:
                 return
 
-        if task.status == RUNNING and task.batch_id is not None:
+        if task.status == RUNNING and task.batch_id is not None and new_status != RUNNING:
             for batch_task in self.get_batch_running_tasks(task.batch_id):
                 self.set_status(batch_task, new_status, config)
                 batch_task.batch_id = None
@@ -566,7 +634,7 @@ class SimpleTaskState(object):
         for task in self.get_active_tasks():
             if remove_stakeholders:
                 task.stakeholders.difference_update(workers)
-            task.workers.difference_update(workers)
+            task.workers -= workers
 
     def disable_workers(self, worker_ids):
         self._remove_workers_from_tasks(worker_ids, remove_stakeholders=False)
@@ -830,6 +898,37 @@ class Scheduler(object):
         for task in orphaned_tasks:
             self._state.set_status(task, PENDING)
 
+    @rpc_method()
+    def count_pending(self, worker):
+        worker_id, worker = worker, self._state.get_worker(worker)
+
+        num_pending, num_unique_pending, num_pending_last_scheduled = 0, 0, 0
+        running_tasks = []
+
+        upstream_status_table = {}
+        for task in worker.get_pending_tasks(self._state):
+            if self._upstream_status(task.id, upstream_status_table) == UPSTREAM_DISABLED:
+                continue
+            if task.status == RUNNING:
+                # Return a list of currently running tasks to the client,
+                # makes it easier to troubleshoot
+                other_worker = self._state.get_worker(task.worker_running)
+                if other_worker is not None:
+                    more_info = {'task_id': task.id, 'worker': str(other_worker)}
+                    more_info.update(other_worker.info)
+                    running_tasks.append(more_info)
+            elif task.status == PENDING:
+                num_pending += 1
+                num_unique_pending += int(len(task.workers) == 1)
+                num_pending_last_scheduled += int(task.workers.peek(last=True) == worker_id)
+        return {
+            'n_pending_tasks': num_pending,
+            'n_unique_pending': num_unique_pending,
+            'n_pending_last_scheduled': num_pending_last_scheduled,
+            'worker_state': worker.state,
+            'running_tasks': running_tasks,
+        }
+
     @rpc_method(allow_null=False)
     def get_work(self, host=None, assistant=False, current_tasks=None, worker=None, **kwargs):
         # TODO: remove any expired nodes
@@ -878,12 +977,7 @@ class Scheduler(object):
             # batch running tasks that weren't claimed since the last get_work go back in the pool
             self._reset_orphaned_batch_running_tasks(worker_id)
 
-        locally_pending_tasks = 0
-        running_tasks = []
-        upstream_table = {}
-
         greedy_resources = collections.defaultdict(int)
-        n_unique_pending = 0
 
         worker = self._state.get_worker(worker_id)
         if worker.is_trivial_worker(self._state):
@@ -901,23 +995,6 @@ class Scheduler(object):
         tasks.sort(key=self._rank, reverse=True)
 
         for task in tasks:
-            in_workers = (assistant and getattr(task, 'runnable', bool(task.workers))) or worker_id in task.workers
-            if task.status == RUNNING and in_workers:
-                # Return a list of currently running tasks to the client,
-                # makes it easier to troubleshoot
-                other_worker = self._state.get_worker(task.worker_running)
-                more_info = {'task_id': task.id, 'worker': str(other_worker)}
-                if other_worker is not None:
-                    more_info.update(other_worker.info)
-                    running_tasks.append(more_info)
-
-            if task.status == PENDING and in_workers:
-                upstream_status = self._upstream_status(task.id, upstream_table)
-                if upstream_status != UPSTREAM_DISABLED:
-                    locally_pending_tasks += 1
-                    if len(task.workers) == 1 and not assistant:
-                        n_unique_pending += 1
-
             if (best_task and batched_params and task.family == best_task.family and
                     len(batched_tasks) < max_batch_size and task.is_batchable() and all(
                     task.params.get(name) == value for name, value in unbatched_params.items()) and
@@ -934,6 +1011,7 @@ class Scheduler(object):
                     greedy_resources[resource] += amount
 
             if self._schedulable(task) and self._has_resources(task.resources, greedy_resources):
+                in_workers = (assistant and task.runnable) or worker_id in task.workers
                 if in_workers and self._has_resources(task.resources, used_resources):
                     best_task = task
                     batch_param_names, max_batch_size = self._state.get_batcher(
@@ -963,12 +1041,7 @@ class Scheduler(object):
 
                             break
 
-        reply = {'n_pending_tasks': locally_pending_tasks,
-                 'running_tasks': running_tasks,
-                 'task_id': None,
-                 'n_unique_pending': n_unique_pending,
-                 'worker_state': worker.state,
-                 }
+        reply = self.count_pending(worker_id)
 
         if len(batched_tasks) > 1:
             batch_string = '|'.join(task.id for task in batched_tasks)
@@ -997,6 +1070,9 @@ class Scheduler(object):
             reply['task_module'] = getattr(best_task, 'module', None)
             reply['task_params'] = best_task.params
 
+        else:
+            reply['task_id'] = None
+
         return reply
 
     @rpc_method(attempts=1)
