@@ -94,6 +94,16 @@ class TaskException(Exception):
     pass
 
 
+GetWorkResponse = collections.namedtuple('GetWorkResponse', (
+    'task_id',
+    'running_tasks',
+    'n_pending_tasks',
+    'n_unique_pending',
+    'n_pending_last_scheduled',
+    'worker_state',
+))
+
+
 class TaskProcess(multiprocessing.Process):
 
     """ Wrap all task execution in this class.
@@ -101,17 +111,17 @@ class TaskProcess(multiprocessing.Process):
     Mainly for convenience since this is run in a separate process. """
 
     def __init__(self, task, worker_id, result_queue, tracking_url_callback,
-                 status_message_callback, random_seed=False, worker_timeout=0):
+                 status_message_callback, use_multiprocessing=False, worker_timeout=0):
         super(TaskProcess, self).__init__()
         self.task = task
         self.worker_id = worker_id
         self.result_queue = result_queue
         self.tracking_url_callback = tracking_url_callback
         self.status_message_callback = status_message_callback
-        self.random_seed = random_seed
         if task.worker_timeout is not None:
             worker_timeout = task.worker_timeout
         self.timeout_time = time.time() + worker_timeout if worker_timeout else None
+        self.use_multiprocessing = use_multiprocessing or self.timeout_time is not None
 
     def _run_get_new_deps(self):
         self.task.set_tracking_url = self.tracking_url_callback
@@ -146,7 +156,7 @@ class TaskProcess(multiprocessing.Process):
     def run(self):
         logger.info('[pid %s] Worker %s running   %s', os.getpid(), self.worker_id, self.task)
 
-        if self.random_seed:
+        if self.use_multiprocessing:
             # Need to have different random seeds if running in separate processes
             random.seed((os.getpid(), time.time()))
 
@@ -310,6 +320,10 @@ class worker(Config):
                                   description='worker-count-uniques means that we will keep a '
                                   'worker alive only if it has a unique pending task, as '
                                   'well as having keep-alive true')
+    count_last_scheduled = BoolParameter(default=False,
+                                         description='Keep a worker alive only if there are '
+                                                     'pending tasks which it was the last to '
+                                                     'schedule.')
     wait_interval = FloatParameter(default=1.0,
                                    config_path=dict(section='core', name='worker-wait-interval'))
     wait_jitter = FloatParameter(default=5.0)
@@ -410,11 +424,7 @@ class Worker(object):
                 pass
 
         # Keep info about what tasks are running (could be in other processes)
-        if worker_processes == 1:
-            self._task_result_queue = DequeQueue()
-        else:
-            self._task_result_queue = multiprocessing.Queue()
-
+        self._task_result_queue = multiprocessing.Queue()
         self._running_tasks = {}
 
         # Stuff for execution_summary
@@ -536,6 +546,21 @@ class Worker(object):
         message = notifications.format_task_error(headline, task, command, formatted_traceback)
         notifications.send_error_email(formatted_subject, message, task.owner_email)
 
+    def _handle_task_load_error(self, exception, task_ids):
+        msg = 'Cannot find task(s) sent by scheduler: {}'.format(','.join(task_ids))
+        logger.exception(msg)
+        subject = 'Luigi: {}'.format(msg)
+        error_message = notifications.wrap_traceback(exception)
+        for task_id in task_ids:
+            self._add_task(
+                worker=self._id,
+                task_id=task_id,
+                status=FAILED,
+                runnable=False,
+                expl=error_message,
+            )
+        notifications.send_error_email(subject, error_message)
+
     def add(self, task, multiprocess=False):
         """
         Add a Task for the worker to check and possibly schedule and run.
@@ -698,26 +723,40 @@ class Worker(object):
         self._worker_info.append(('first_task', self._first_task))
         self._scheduler.add_worker(self._id, self._worker_info)
 
-    def _log_remote_tasks(self, running_tasks, n_pending_tasks, n_unique_pending):
+    def _log_remote_tasks(self, get_work_response):
         logger.debug("Done")
         logger.debug("There are no more tasks to run at this time")
-        if running_tasks:
-            for r in running_tasks:
+        if get_work_response.running_tasks:
+            for r in get_work_response.running_tasks:
                 logger.debug('%s is currently run by worker %s', r['task_id'], r['worker'])
-        elif n_pending_tasks:
-            logger.debug("There are %s pending tasks possibly being run by other workers", n_pending_tasks)
-            if n_unique_pending:
-                logger.debug("There are %i pending tasks unique to this worker", n_unique_pending)
+        elif get_work_response.n_pending_tasks:
+            logger.debug(
+                "There are %s pending tasks possibly being run by other workers",
+                get_work_response.n_pending_tasks)
+            if get_work_response.n_unique_pending:
+                logger.debug(
+                    "There are %i pending tasks unique to this worker",
+                    get_work_response.n_unique_pending)
+            if get_work_response.n_pending_last_scheduled:
+                logger.debug(
+                    "There are %i pending tasks last scheduled by this worker",
+                    get_work_response.n_pending_last_scheduled)
 
     def _get_work_task_id(self, get_work_response):
-        if get_work_response['task_id'] is not None:
+        if get_work_response.get('task_id') is not None:
             return get_work_response['task_id']
         elif 'batch_id' in get_work_response:
-            task = load_task(
-                module=get_work_response.get('task_module'),
-                task_name=get_work_response['task_family'],
-                params_str=get_work_response['task_params'],
-            )
+            try:
+                task = load_task(
+                    module=get_work_response.get('task_module'),
+                    task_name=get_work_response['task_family'],
+                    params_str=get_work_response['task_params'],
+                )
+            except Exception as ex:
+                self._handle_task_load_error(ex, get_work_response['batch_task_ids'])
+                self.run_succeeded = False
+                return None
+
             self._scheduler.add_task(
                 worker=self._id,
                 task_id=task.task_id,
@@ -728,23 +767,26 @@ class Worker(object):
                 batch_id=get_work_response['batch_id'],
             )
             return task.task_id
+        else:
+            return None
 
     def _get_work(self):
         if self._stop_requesting_work:
-            return None, 0, 0, 0, WORKER_STATE_DISABLED
-        logger.debug("Asking scheduler for work...")
-        r = self._scheduler.get_work(
-            worker=self._id,
-            host=self.host,
-            assistant=self._assistant,
-            current_tasks=list(self._running_tasks.keys()),
-        )
-        n_pending_tasks = r['n_pending_tasks']
+            return GetWorkResponse(None, 0, 0, 0, 0, WORKER_STATE_DISABLED)
+
+        if self.worker_processes > 0:
+            logger.debug("Asking scheduler for work...")
+            r = self._scheduler.get_work(
+                worker=self._id,
+                host=self.host,
+                assistant=self._assistant,
+                current_tasks=list(self._running_tasks.keys()),
+            )
+        else:
+            logger.debug("Checking if tasks are still pending")
+            r = self._scheduler.count_pending(worker=self._id)
+
         running_tasks = r['running_tasks']
-        n_unique_pending = r['n_unique_pending']
-        # TODO: For a tiny amount of time (a month?) we'll keep forwards compatibility
-        # That is you can user a newer client than server (Sep 2016)
-        worker_state = r.get('worker_state', WORKER_STATE_ACTIVE)  # state according to server!
         task_id = self._get_work_task_id(r)
 
         self._get_work_response_history.append({
@@ -762,13 +804,7 @@ class Worker(object):
                               task_name=r['task_family'],
                               params_str=r['task_params'])
             except TaskClassException as ex:
-                msg = 'Cannot find task for %s' % task_id
-                logger.exception(msg)
-                subject = 'Luigi: %s' % msg
-                error_message = notifications.wrap_traceback(ex)
-                notifications.send_error_email(subject, error_message)
-                self._add_task(worker=self._id, task_id=task_id, status=FAILED, runnable=False,
-                               assistant=self._assistant)
+                self._handle_task_load_error(ex, [task_id])
                 task_id = None
                 self.run_succeeded = False
 
@@ -777,21 +813,31 @@ class Worker(object):
                 self._scheduled_tasks.get(batch_id) for batch_id in r['batch_task_ids']])
             self._batch_running_tasks[task_id] = batch_tasks
 
-        return task_id, running_tasks, n_pending_tasks, n_unique_pending, worker_state
+        return GetWorkResponse(
+            task_id=task_id,
+            running_tasks=running_tasks,
+            n_pending_tasks=r['n_pending_tasks'],
+            n_unique_pending=r['n_unique_pending'],
+
+            # TODO: For a tiny amount of time (a month?) we'll keep forwards compatibility
+            #  That is you can user a newer client than server (Sep 2016)
+            n_pending_last_scheduled=r.get('n_pending_last_scheduled', 0),
+            worker_state=r.get('worker_state', WORKER_STATE_ACTIVE),
+        )
 
     def _run_task(self, task_id):
         task = self._scheduled_tasks[task_id]
 
-        p = self._create_task_process(task)
+        task_process = self._create_task_process(task)
 
-        self._running_tasks[task_id] = p
+        self._running_tasks[task_id] = task_process
 
-        if self.worker_processes > 1:
+        if task_process.use_multiprocessing:
             with fork_lock:
-                p.start()
+                task_process.start()
         else:
             # Run in the same process
-            p.run()
+            task_process.run()
 
     def _create_task_process(self, task):
         def update_tracking_url(tracking_url):
@@ -807,7 +853,7 @@ class Worker(object):
 
         return TaskProcess(
             task, self._id, self._task_result_queue, update_tracking_url, update_status_message,
-            random_seed=bool(self.worker_processes > 1),
+            use_multiprocessing=bool(self.worker_processes > 1),
             worker_timeout=self._config.timeout
         )
 
@@ -908,7 +954,7 @@ class Worker(object):
             time.sleep(wait_interval)
             yield
 
-    def _keep_alive(self, n_pending_tasks, n_unique_pending):
+    def _keep_alive(self, get_work_response):
         """
         Returns true if a worker should stay alive given.
 
@@ -923,8 +969,12 @@ class Worker(object):
             return False
         elif self._assistant:
             return True
+        elif self._config.count_last_scheduled:
+            return get_work_response.n_pending_last_scheduled > 0
+        elif self._config.count_uniques:
+            return get_work_response.n_unique_pending > 0
         else:
-            return n_pending_tasks and (n_unique_pending or not self._config.count_uniques)
+            return get_work_response.n_pending_tasks > 0
 
     def handle_interrupt(self, signum, _):
         """
@@ -953,20 +1003,20 @@ class Worker(object):
         self._add_worker()
 
         while True:
-            while len(self._running_tasks) >= self.worker_processes:
+            while len(self._running_tasks) >= self.worker_processes > 0:
                 logger.debug('%d running tasks, waiting for next task to finish', len(self._running_tasks))
                 self._handle_next_task()
 
-            task_id, running_tasks, n_pending_tasks, n_unique_pending, worker_state = self._get_work()
+            get_work_response = self._get_work()
 
-            if worker_state == WORKER_STATE_DISABLED:
+            if get_work_response.worker_state == WORKER_STATE_DISABLED:
                 self._start_phasing_out()
 
-            if task_id is None:
+            if get_work_response.task_id is None:
                 if not self._stop_requesting_work:
-                    self._log_remote_tasks(running_tasks, n_pending_tasks, n_unique_pending)
+                    self._log_remote_tasks(get_work_response)
                 if len(self._running_tasks) == 0:
-                    if self._keep_alive(n_pending_tasks, n_unique_pending):
+                    if self._keep_alive(get_work_response):
                         six.next(sleeper)
                         continue
                     else:
@@ -976,8 +1026,8 @@ class Worker(object):
                     continue
 
             # task_id is not None:
-            logger.debug("Pending tasks: %s", n_pending_tasks)
-            self._run_task(task_id)
+            logger.debug("Pending tasks: %s", get_work_response.n_pending_tasks)
+            self._run_task(get_work_response.task_id)
 
         while len(self._running_tasks):
             logger.debug('Shut down Worker, %d more tasks to go', len(self._running_tasks))
